# (c) Copyright 2005, 2008, 2010. CodeWeavers, Inc.
package CXMenuSUSE8;
use warnings;
use strict;
use CXLog;
use CXUtils;
use CXMenuKG;
use base "CXMenuKG";

# Notes:
# * On SuSE 8, SuSEconfig mishandles the '$$' sequence when generating the
#   Gnome menus. It also ignores some menus for no discernable reason.


sub detect($$$$)
{
    my ($class, $cxoptions, $cxconfig, $gui_info)=@_;
    return () if (!$gui_info->{suse_on});

    my @selves;
    if ($gui_info->{suse_preferred_menu})
    {
        my $self={
            tag             => $cxoptions->{tag},
            encoding        => "UTF-8",
            escaping        => "dollar",
            menu            => $gui_info->{suse_preferred_menu},
            global_menu     => $gui_info->{suse_global_menu},
            extension       => "desktop",
            extensions      => "\\.desktop",
            section_name    => "Desktop Entry"
        };
        bless $self, $class;
        push @selves, $self;
    }

    return @selves;
}

sub needs_icon_hack($)
{
    my ($self)=@_;

    my $icon_hack=$self->{icon_hack};
    if (!defined $icon_hack)
    {
        # This is a *very ugly* hack to work around a SuSE 8.1 and 8.2 bug.
        # On SuSE 8.1 the susewm script transforms our icon path from
        # Icon=$CX_ROOT/share/icons/crossover.xpm to
        # Icon=$CX_ROOT/share/icons/crossover for KDE so that KDE
        # does not find our icon anymore, and
        # Icon=$CX_ROOT/share/icons/crossover.png for Gnome so that
        # we need to create a PNG file for Gnome. The workaround is to add
        # an extra '.xpm' extension to the icon name, and create the
        # corresponding .png file.
        #
        # On SuSE 8.2, susewm does the same kind of transformations but only
        # for menu items, not for menu folders!
        #
        # SuSE 8.0 (more precisely susewm-3.6.0-74) also changes the icon
        # path to point to a PNG file for Gnome, but leaves the path
        # intact for KDE.
        #
        # Finally, note that we cannot call 'rpm -q' to check the susewm
        # version because this does not work when called from an RPM's
        # postinstall script (due to the RPM database being locked).
        $self->{icon_hack}=0;

        my $filename="/etc/SuSE-release";
        if (-f $filename)
        {
            if (open(my $fh, "<", $filename))
            {
                cxlog("Reading '$filename'\n");
                while (my $line=<$fh>)
                {
                    cxlog($line);
                    if ($line =~ /^SuSE Linux (8\.[12])\b/)
                    {
                        $self->{icon_hack}=$1;
                        cxlog(" -> activating the SuSE $self->{icon_hack} icon hack\n");
                        last;
                    }
                }
                close($fh);
            }
            else
            {
                cxwarn("unable to open '$filename' for reading: $!\n");
            }
        }
        $icon_hack=$self->{icon_hack};
    }
    return $icon_hack;
}

sub install($$)
{
    my ($self, $components)=@_;

    return 1 if ($components->[-1]->{is_desktop});

    my $icon_hack;
    if ($> == 0)
    {
        $icon_hack=$self->needs_icon_hack();
        foreach my $component (@$components)
        {
            $component->{_icon}=$component->{icon};
            if ($icon_hack eq "8.1" or
                ($icon_hack eq "8.2" and !$component->{is_dir}))
            {
                $component->{icon}.=".xpm";
            }
            my $png=$component->{icon};
            $png=~s/\.xpm$/.png/;
            CXUtils::convert_to_png($component->{_icon}, $png);
        }
    }
    my $rc=$self->SUPER::install($components);
    map { $_->{icon}=$_->{_icon}; } @$components if ($icon_hack);
    return $rc;
}

sub finalize($)
{
    my ($self)=@_;

    my $rc=$self->SUPER::finalize();
    if ($self->{modified} and $self->{menu} eq $self->{global_menu})
    {
        # Make sure we only run one instance at a time (just to be safe).
        my $lock=CXUtils::cxlock("susewm");
        cxsystem("SuSEconfig --module susewm >/dev/null");
        CXUtils::cxunlock($lock);
    }
    return $rc;
}

return 1;
