# (c) Copyright 2010. CodeWeavers, Inc.

import os
import traceback
import getpass

import gtk
import gtk.glade
import gobject

import cxutils

import distversion
import pyop
import bottlemanagement
import cxguitools

from cxutils import cxgettext as _

# Portable which(1) implementation
def which(path, app):
    """Looks for an executable in the specified directory list.

    path is an os.pathsep-separated list of directories and app is the
    executable name. If app contains a path separator then path is ignored.
    If the file is not found, then None is returned.
    """
    if os.path.isabs(app):
        if os.path.isfile(app) and os.access(app, os.X_OK):
            return app
    elif os.sep in app or (os.altsep and os.altsep in app):
        app_path = os.path.join(os.getcwd(), app)
        if os.path.isfile(app_path) and os.access(app_path, os.X_OK):
            return app_path
    else:
        for directory in path.split(os.pathsep):
            if directory == "":
                continue
            app_path = os.path.join(directory, app)
            if os.path.isfile(app_path) and os.access(app_path, os.X_OK):
                return app_path
    return None


class PackageBottleController(object):

    xml = None
    folder_model = None

    def __init__(self, bottleobj):

        self.bottle = bottleobj

        try:
            gladepath = os.environ["CX_GLADEPATH"]
        except KeyError:
            gladepath = os.path.join(cxutils.CX_ROOT, "lib", "python", "glade")

        self.gladefile = os.path.join(gladepath,"packagedlg.glade")

        if not gtk.glade.textdomain() == "crossover":
            locale_path = os.path.join(cxutils.CX_ROOT, "share", "locale")
            gtk.glade.bindtextdomain("crossover", locale_path)
            gtk.glade.textdomain("crossover")
        self.xml = gtk.glade.XML(self.gladefile)
        self.xml.signal_autoconnect(self)
        self.progbar = self.xml.get_widget("progbar")

        # fill in dialog defaults
        self.xml.get_widget("release_entry").set_text("1")
        self.xml.get_widget("packager_entry").set_text(_("Bottle Packager <%s>") % getpass.getuser())
        self.xml.get_widget("product_id_entry").set_text(distversion.BUILTIN_PRODUCT_ID)

        self.xml.get_widget("package_type_combo").append_text("deb")
        self.xml.get_widget("package_type_combo").append_text("rpm")
        self.xml.get_widget("package_type_combo").append_text("sunpkg")

        # Try to select a reasonable default for the package type
        dpkg = which(os.environ["PATH"], "dpkg")
        rpm = which(os.environ["PATH"], "rpm")


        if (dpkg != ""):
            self.xml.get_widget("package_type_combo").set_active(0)
        elif (rpm != ""):
            self.xml.get_widget("package_type_combo").set_active(1)
        else:
            self.xml.get_widget("package_type_combo").set_active(2)

        self.xml.get_widget("package_bottle_dialog").set_title(_("Create Package From %s") % self.bottle.name)
        self.xml.get_widget("package_bottle_dialog").show()

    def cancel_clicked(self, caller):
        self.xml.get_widget("package_bottle_dialog").destroy()
        self.quit_requested(caller)

    def progbar_pulse(self):
        self.progbar.pulse()
        return True

    def create_clicked(self, _widget):
        self.animateEvent = gobject.timeout_add(100, self.progbar_pulse)

        release = self.xml.get_widget("release_entry").get_text()
        packager = self.xml.get_widget("packager_entry").get_text()
        productID = self.xml.get_widget("product_id_entry").get_text()
        pkg_type = self.xml.get_widget("package_type_combo").get_active_text()
        outdir = self.xml.get_widget("location_chooser").get_filename()

        self.packageOp = PackageBottleOperation(self, pkg_type, self.bottle.name, packager, release, productID, outdir)

        self.xml.get_widget("package_details").set_sensitive(False)
        self.xml.get_widget("package_type_combo").set_sensitive(False)
        self.xml.get_widget("dialog-action_area").set_sensitive(False)
        self.progbar.show()

        pyop.sharedOperationQueue.enqueue(self.packageOp)


    def quit_requested(self, _caller):
        cxguitools.toplevel_quit()

    def package_command_finished(self, _caller, retcode, err):
        if retcode:
            self.xml.get_widget("package_bottle_dialog").destroy()
            self.quit_requested(None)
        else:
            self.xml.get_widget("package_details").set_sensitive(True)
            self.xml.get_widget("package_type_combo").set_sensitive(True)
            self.xml.get_widget("dialog-action_area").set_sensitive(True)
            self.progbar.hide()

            title = _("Packaging failed")
            cxguitools.CXMessageDlg(primary=title, secondary=err, message_type=gtk.MESSAGE_ERROR, parent=self.xml.get_widget("package_bottle_dialog"))


class PackageBottleOperation(pyop.PythonOperation):

    def __init__(self, inPackageController, inType, inBottleName, inPackager, inRelease, inProductID, inOutDir):
        pyop.PythonOperation.__init__(self)

        self.bottlename = inBottleName
        self.packageController = inPackageController
        self.type = inType
        self.packager = inPackager
        self.release = inRelease
        self.productID = inProductID
        self.outDir = inOutDir


    def __unicode__(self):
        return "PackageBottleOperation for " + self.bottlename


    def main(self):
        try:
            self.retcode, self.err = bottlemanagement.package_bottle(self.bottlename, self.type, self.release, self.productID, self.packager, self.outDir)
        except:
            traceback.print_exc()


    def finish(self):
        self.packageController.package_command_finished(self, self.retcode, self.err)
        pyop.PythonOperation.finish(self)


    
