# (c) Copyright 2009-2010. CodeWeavers, Inc.

import os
import subprocess
import traceback

import gobject
import gtk
import gtk.glade
gtk.gdk.threads_init()

import cxlog
import cxutils
import bottlequery

import bottlecollection
import createshortcutdlg
import cxguitools
import pyop

# for localization
from cxutils import cxgettext as _

class RunCommandController(object):

    mainWindow = None
    xml = None

    modeldialogxml = None


    def __init__(self, inBottleName):

        #----------------------------------------------------------------------------
        #  Setup the GUI
        #----------------------------------------------------------------------------

        try:
            gladepath = os.environ["CX_GLADEPATH"]
        except KeyError:
            gladepath = os.path.join(cxutils.CX_ROOT, "lib", "python", "glade")

        self.gladefile = os.path.join(gladepath,"runcommanddlg.glade")

        if not gtk.glade.textdomain() == "crossover":
            locale_path = os.path.join(cxutils.CX_ROOT, "share", "locale")
            gtk.glade.bindtextdomain("crossover", locale_path)
            gtk.glade.textdomain("crossover")
        self.xml = gtk.glade.XML(self.gladefile)
        self.xml.signal_autoconnect(self)

        try:
            self.xml.get_widget("RunCommandDialog").set_icon_from_file(
                os.path.join(cxutils.CX_ROOT, 'share', 'icons', 'cxrun.xpm'))
        except gobject.GError:
            cxlog.warn("Couldn't load icon:\n%s" % traceback.format_exc())


        #----------------------------------------------------------------------------
        #  Set up progress bar
        #----------------------------------------------------------------------------
        progbar = self.xml.get_widget("ProgBar")
        progbar.set_pulse_step(.05)
        progbar.hide()


        #----------------------------------------------------------------------------
        #  Fill Bottle popup list
        #----------------------------------------------------------------------------
        bottlePopupWidget = self.xml.get_widget("BottlePopup")
        self.bottleListStore = gtk.ListStore(str)
        bottlePopupWidget.set_model(self.bottleListStore)
        selectIter = 0
        collection = bottlecollection.sharedCollection()
        if len(collection.bottleList()) > 0:
            for bottleName in sorted(collection.bottleList()):
                newrow = self.bottleListStore.append()
                self.bottleListStore.set_value(newrow, 0, bottleName)
                if bottleName == inBottleName:
                    selectIter = newrow
        else:
            newrow = self.bottleListStore.append()
            self.bottleListStore.set_value(newrow, 0, _("(No bottles)"))

        if selectIter:
            bottlePopupWidget.set_active_iter(selectIter)
        else:
            bottlePopupWidget.set_active_iter(self.bottleListStore.get_iter_first())

        self.commandOp = None
        self.set_widget_sensitivities(self)

    def set_command(self, command):
        commandEntry = self.xml.get_widget("CommandEntry")
        commandEntry.set_text(command)

    def cancel_clicked(self, caller):
        self.xml.get_widget("RunCommandDialog").destroy()
        self.quit_requested(caller)

    def quit_requested(self, _caller):
        cxguitools.toplevel_quit()

    def progbar_pulse(self):
        progBar = self.xml.get_widget("ProgBar")
        progBar.pulse()
        return True

    def set_widget_sensitivities(self, _caller):
        progBar = self.xml.get_widget("ProgBar")

        collection = bottlecollection.sharedCollection()
        if self.commandOp != None or len(collection.bottleList()) == 0:
            self.xml.get_widget("RunCommandButton").set_sensitive(False)
            self.xml.get_widget("CreateIconButton").set_sensitive(False)
            self.xml.get_widget("CommandEntry").set_sensitive(False)
            self.xml.get_widget("BottlePopup").set_sensitive(False)
            self.xml.get_widget("DebugOptionsFrame").set_sensitive(False)
            self.xml.get_widget("CommandBrowseButton").set_sensitive(False)
            progBar.show()
        else:
            if self.xml.get_widget("CommandEntry").get_text():
                self.xml.get_widget("RunCommandButton").set_sensitive(True)
                self.xml.get_widget("CreateIconButton").set_sensitive(True)
            else:
                self.xml.get_widget("RunCommandButton").set_sensitive(False)
                self.xml.get_widget("CreateIconButton").set_sensitive(False)

            self.xml.get_widget("CommandEntry").set_sensitive(True)
            self.xml.get_widget("BottlePopup").set_sensitive(True)
            self.xml.get_widget("DebugOptionsFrame").set_sensitive(True)
            self.xml.get_widget("CommandBrowseButton").set_sensitive(True)
            progBar.hide()

        if len(collection.bottleList()) == 0:
            self.xml.get_widget("ProgBar").hide()
        

    def run_command(self, _caller):

        self.logFileName = ""
        loggingChannels = ""
        if self.xml.get_widget("DebugExpander").get_expanded() and self.xml.get_widget("LogFileCheckbox").get_active():
            fileSaver = gtk.FileChooserDialog(_("Specify where to save the log file"), self.xml.get_widget("RunCommandDialog"), gtk.FILE_CHOOSER_ACTION_SAVE)
            fileFilter = gtk.FileFilter()
            fileFilter.add_pattern("*.cxlog")
            fileFilter.set_name(_("CrossOver Log Files"))
            fileSaver.add_filter(fileFilter)

            fileSaver.add_button(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL)
            fileSaver.add_button(gtk.STOCK_SAVE, gtk.RESPONSE_OK)
            fileSaver.set_do_overwrite_confirmation(True)

            if fileSaver.run() == gtk.RESPONSE_OK:
                self.logFileName = fileSaver.get_filename()
                fileSaver.destroy()
            else:
                fileSaver.destroy()
                return

            if self.xml.get_widget("DbgChannelRelay").get_active():
                loggingChannels += ",+relay"
            if self.xml.get_widget("DbgChannelSeh").get_active():
                loggingChannels += ",+seh"
            if self.xml.get_widget("DbgChannelTid").get_active():
                loggingChannels += ",+tid"
            if self.xml.get_widget("DbgChannelThread").get_active():
                loggingChannels += ",+thread"
            if self.xml.get_widget("DbgChannelPrint").get_active():
                loggingChannels += ",+print"
            if self.xml.get_widget("DbgChannelPsdrv").get_active():
                loggingChannels += ",+psdrv"
            if self.xml.get_widget("DbgChannelFont").get_active():
                loggingChannels += ",+font"
            if self.xml.get_widget("DbgChannelFile").get_active():
                loggingChannels += ",+file"
            if self.xml.get_widget("DbgChannelKey").get_active():
                loggingChannels += ",+key"
            if self.xml.get_widget("DbgChannelKeyboard").get_active():
                loggingChannels += ",+keyboard"
            if self.xml.get_widget("DbgChannelX11drv").get_active():
                loggingChannels += ",+x11drv"
            if self.xml.get_widget("DbgChannelEvent").get_active():
                loggingChannels += ",+event"
            loggingChannels += "," + (self.xml.get_widget("DbgChannelOther").get_text())

            loggingChannels = loggingChannels.lstrip(",")


        self.animateEvent = gobject.timeout_add(100, self.progbar_pulse)

        commandEntry = self.xml.get_widget("CommandEntry")
        self.commandOp = RunWineCommandOperation(self, commandEntry.get_text(), self.get_selected_bottle().name, self.logFileName, loggingChannels)
        self.set_widget_sensitivities(self)
        pyop.sharedOperationQueue.enqueue(self.commandOp)


    def toggle_logfile(self, widget):
        self.xml.get_widget("ExtraLoggingChannelsFrame").set_sensitive(
            widget.get_active())


    def run_command_finished(self, _runCommandOp):
        self.commandOp = None
        self.set_widget_sensitivities(self)

        gobject.source_remove(self.animateEvent)


    def get_selected_bottle(self):
        collection = bottlecollection.sharedCollection()
        bottlePopup = self.xml.get_widget("BottlePopup")
        activeIndex = bottlePopup.get_active_iter()
        selectedBottleName = self.bottleListStore.get_value(activeIndex, 0)
        bottleObj = collection.bottleObject(selectedBottleName)
        return bottleObj


    def browse_for_command(self, _caller):
        selectedBottle = self.get_selected_bottle()
        filePicker = gtk.FileChooserDialog(_("Choose a File to Run"), self.xml.get_widget("RunCommandDialog"), gtk.FILE_CHOOSER_ACTION_OPEN)
        cxguitools.add_filters(filePicker, cxguitools.FILTERS_RUNNABLE | cxguitools.FILTERS_ALLFILES)

        environ = bottlequery.get_win_environ(selectedBottle.name, ("SystemDrive",))
        drive = bottlequery.expand_win_string(environ, "%SystemDrive%")
        drive = bottlequery.get_native_path(selectedBottle.name, drive)
        filePicker.set_current_folder(drive)

        filePicker.add_button(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL)
        filePicker.add_button(gtk.STOCK_OPEN, gtk.RESPONSE_OK)

        if filePicker.run() == gtk.RESPONSE_OK:
            commandEntry =  self.xml.get_widget("CommandEntry")
            filename = cxutils.argvtocmdline((filePicker.get_filename(),))
            commandEntry.set_text(filename)

        filePicker.destroy()

    def create_icon(self, _widget):
        bottlename = self.get_selected_bottle().name
        
        command = self.xml.get_widget("CommandEntry").get_text()

        createshortcutdlg.CreateShortcutDialog(bottlename=bottlename, command=command)


class RunWineCommandOperation(pyop.PythonOperation):

    def __init__(self, inCommandController, inCommand, inBottleName, inLogFile="", inDebugChannels=""):
        pyop.PythonOperation.__init__(self)
        self.bottlename = inBottleName
        self.commandController = inCommandController
        self.argArray = [os.path.join(cxutils.CX_ROOT, "bin", "cxstart")]
        self.argArray.extend(["--bottle", inBottleName])
        if inLogFile != "":
            self.argArray.append("--cx-log")
            self.argArray.append(inLogFile)
        if inDebugChannels != "":
            self.argArray.append("--debugmsg")
            self.argArray.append(inDebugChannels)
        cmdline_array = cxutils.cmdlinetoargv(inCommand)
        dirname = cxutils.dirname(cmdline_array[0])
        if dirname:
            self.argArray.append('--workdir')
            self.argArray.append(dirname)
        self.argArray.append("--")
        self.argArray.extend(cxutils.cmdlinetoargv(inCommand))

    def __unicode__(self):
        return "RunWineCommandOperation for " + self.bottlename


    def main(self):
        try:
            subp = subprocess.Popen(self.argArray, stdout=subprocess.PIPE, stderr=subprocess.PIPE, universal_newlines=True, close_fds=True)
            self.response = subp.communicate()
        except:
            traceback.print_exc()


    def finish(self):
        self.commandController.run_command_finished(self)
        pyop.PythonOperation.finish(self)


